package com.flyfish.framework.query;

import com.flyfish.framework.domain.base.Domain;
import com.flyfish.framework.query.spi.QueryChainFactory;
import com.flyfish.framework.query.support.DomainFunction;
import org.springframework.core.io.support.SpringFactoriesLoader;
import org.springframework.dao.InvalidDataAccessApiUsageException;

import java.util.List;

/**
 * 查询工具类
 * 基于Fluent API风格实现
 * 底层基于适配器兼容各种数据库,包括关系型数据库和
 * 自动判定空值，空值默认情况不会进行查询
 *
 * @author wangyu
 */
public final class Queries {

    @SuppressWarnings("rawtypes")
    private static final List<QueryChainFactory> FACTORIES =
            SpringFactoriesLoader.loadFactories(QueryChainFactory.class, null);

    /**
     * 内部带参工厂函数，调用工厂方法创建实例
     *
     * @param type 工厂类型，最终产出类型需匹配
     * @param <C>  产出查询链泛型
     */
    private static <C extends QueryChain<C, ?>> C produce(Class<?> type) {
        QueryChainFactory<C> factory = getFactory(type);
        return factory.produce();
    }

    /**
     * 创建基于字符串字段名的查询
     *
     * @param column 列名
     * @return 结果
     */
    public static QueryCondition<NamedQueryChain> where(String column) {
        NamedQueryChain chain = produce(NamedQueryChain.class);
        return chain.and(column);
    }

    /**
     * 创建基于lambda字段引用的查询
     *
     * @param getter 列引用
     * @return 结果
     */
    public static <T extends Domain> QueryCondition<LambdaQueryChain<T>> where(DomainFunction<T> getter) {
        LambdaQueryChain<T> chain = produce(LambdaQueryChain.class);
        return chain.and(getter);
    }

    /**
     * 以嵌套条件开始
     *
     * @param chain 要嵌套的条件
     * @return 结果
     */
    public static NamedQueryChain within(NamedQueryChain chain) {
        NamedQueryChain created = produce(LambdaQueryChain.class);
        return created.and(chain);
    }

    /**
     * 以嵌套条件开始
     *
     * @param chain 要嵌套的条件
     * @return 结果
     */
    public static <T extends Domain> LambdaQueryChain<T> within(LambdaQueryChain<T> chain) {
        LambdaQueryChain<T> created = produce(LambdaQueryChain.class);
        return created.and(chain);
    }

    /**
     * 通过特定产出类获取工厂
     *
     * @param targetType 目标类型
     * @return 匹配的工厂
     */
    @SuppressWarnings("unchecked")
    private static <C extends QueryChain<C, ?>> QueryChainFactory<C> getFactory(Class<?> targetType) {
        return FACTORIES.stream()
                .filter(factory -> factory.supports(targetType))
                .findFirst()
                .orElseThrow(() -> new InvalidDataAccessApiUsageException("未实现当前查询的适配工厂！"));
    }

    /**
     * 传入多个查询列表时的连接方式
     */
    public enum Combinator {

        AND, OR
    }

    /**
     * 方向，用于描述查询和部分语法
     */
    public enum Direction {

        LEFT, RIGHT, ALL
    }
}
