package com.flyfish.framework.mongodb.query.impl;

import com.flyfish.framework.query.NamedQueryChain;
import com.flyfish.framework.query.Queries;
import com.flyfish.framework.query.QueryChain;
import com.flyfish.framework.query.QueryCondition;
import lombok.RequiredArgsConstructor;

import java.util.Collection;
import java.util.List;
import java.util.function.Supplier;

/**
 * mongodb实现的基于名称的查询链
 * 这里需要一个机制，能够将父类上下文带入，并且能够知晓实际查询对象的连接方式
 * 这就需要用到观察者模式，以便延迟执行。将动作作为事件发布到对象里，在具体的执行时机再进行fire。
 * 于是乎，我们便可以优雅而且低耦合的实现链式了。
 *
 * @author wangyu
 */
public class MongoNamedQueryChain extends MongoQueryDefinition implements NamedQueryChain {

    private final QueryChainListener listener = null;

    /**
     * 以且连接下一个字段
     *
     * @param column 列
     * @return 查询条件
     */
    @Override
    public QueryCondition<NamedQueryChain> and(String column) {
        listener.waitFor(criteria -> criteria.andOperator(criteria));
        return new NamedQueryCondition(column);
    }

    /**
     * 直接拼接提供者，此处懒加载，最终build才会执行
     *
     * @param supplier 提供者
     * @return 结果
     */
    @Override
    public <V extends QueryChain<V, ?>> NamedQueryChain and(Supplier<QueryChain<V, ?>> supplier) {
        return null;
    }

    /**
     * 条件列表
     *
     * @param chain 多个条件们
     * @return 结果
     */
    @Override
    public NamedQueryChain and(NamedQueryChain chain) {
        return null;
    }

    /**
     * 多个嵌套子条件列表
     *
     * @param chains     多条链
     * @param combinator 各个链条之间的连接方式
     * @return 结果
     */
    @Override
    public NamedQueryChain and(List<NamedQueryChain> chains, Queries.Combinator combinator) {
        return null;
    }

    /**
     * 以或连接下一个字段
     *
     * @param column 列
     * @return 查询条件
     */
    @Override
    public QueryCondition<NamedQueryChain> or(String column) {
        return null;
    }

    /**
     * 以或连接条件列表
     *
     * @param chain 多个条件们
     * @return 结果
     */
    @Override
    public NamedQueryChain or(NamedQueryChain chain) {
        return null;
    }

    /**
     * 以或连接嵌套多个子条件列表
     *
     * @param chains     多条链
     * @param combinator 各个链条之间的连接方式
     * @return 结果
     */
    @Override
    public NamedQueryChain or(List<NamedQueryChain> chains, Queries.Combinator combinator) {
        return null;
    }

    /**
     * 直接拼接提供者，此处懒加载，最终build才会执行
     *
     * @param supplier 提供者
     * @return 结果
     */
    @Override
    public <V extends QueryChain<V, ?>> NamedQueryChain or(Supplier<QueryChain<V, ?>> supplier) {
        return null;
    }

    /**
     * 条件操作实现
     *
     * @author wangyu
     */
    @RequiredArgsConstructor
    class NamedQueryCondition implements QueryCondition<NamedQueryChain> {

        private final String column;

        /**
         * 相等判定
         *
         * @param value 值
         * @return 结果
         */
        @Override
        public NamedQueryChain eq(Object value) {
            criteria.is(value);
            listener.fire();
            return MongoNamedQueryChain.this;
        }

        /**
         * 判定某列的值中存在指定值，特指json array数据类型且子类型中带有id的场景
         * 用于兼容mongodb查询，mysql查询使用JSON_CONTAINS进行判定。
         * 当且仅当mongodb会拼接.$id，mysql一律匹配id字段
         * <p>
         * 等价于 .eq(value)
         *
         * @param value 值
         * @return 结果
         */
        @Override
        public NamedQueryChain hasId(Object value) {
            return null;
        }

        /**
         * 对于mongodb，自动处理。对于关系型数据库，代表json array中是否包含对应值，值仅支持基本数据类型
         * <p>
         * 等价于 .eq(value)
         *
         * @param value 基本数据类型的值
         * @return 结果
         */
        @Override
        public NamedQueryChain has(Object value) {
            return null;
        }

        /**
         * 不等判定
         *
         * @param value 值
         * @return 结果
         */
        @Override
        public NamedQueryChain ne(Object value) {
            return null;
        }

        /**
         * 值介于两者之间
         *
         * @param items 双值列表
         * @return 结果
         */
        @Override
        public NamedQueryChain between(List<?> items) {
            return null;
        }

        /**
         * 模糊匹配，这里是全模糊
         *
         * @param keyword 查询关键字
         * @return 结果
         */
        @Override
        public NamedQueryChain like(String keyword) {
            return null;
        }

        /**
         * 根据指定的方向进行模糊查询
         *
         * @param keyword   关键字
         * @param direction 方向，可以匹配开头和结尾
         * @return 结果
         */
        @Override
        public NamedQueryChain like(String keyword, Queries.Direction direction) {
            return null;
        }

        /**
         * 判定为空
         *
         * @return 结果
         */
        @Override
        public NamedQueryChain isNull() {
            return null;
        }

        /**
         * 包含在内
         *
         * @param list 集合
         * @return 结果
         */
        @Override
        public NamedQueryChain in(Collection<?> list) {
            return null;
        }

        /**
         * 包含在内
         * 注意，需要根据字段类型推断。
         * 如果是json数组，需要进行双向匹配
         *
         * @param values 值们
         * @return 结果
         */
        @Override
        public NamedQueryChain in(Object... values) {
            return null;
        }
    }
}
